package student

import (
	"bufio"
	"errors"
	"fmt"
	"sync"

	//"github.com/syndtr/goleveldb/leveldb"
	"github.com/tealeg/xlsx"
	"io"
	"os"
	"strings"
	"unicode"
)

func ParseHan(str string) string {
	ret := ""
	for _, r := range str {
		if unicode.Is(unicode.Han, r) {
			ret = ret + string(r)
		}
	}
	return ret
}
func getTag(questionType string) string {
	if strings.Contains(questionType, "单选题") {
		return "1"
	}
	if strings.Contains(questionType, "判断题") {
		return "2"
	}
	if strings.Contains(questionType, "多选题") {
		return "3"
	}
	return ""
}

var CourseMap map[string]int
var CourseMapArray []map[string][]string
var mutex sync.Mutex

func TrimSpace(data string) string {
	str := strings.TrimSpace(data)
	ret := ""
	for _, r := range str {
		if string(r) == " " {
			continue
		}
		if fmt.Sprintf("%x", r) == "a0" {
			continue
		}
		ret = ret + string(r)
	}
	return ret
}

func LoadXlsx(filename string) (map[string][]string, error) {
	mutex.Lock()
	defer mutex.Unlock()
	if CourseMap == nil {
		CourseMap = map[string]int{}
		CourseMapArray = []map[string][]string{}
	}
	if index, ok := CourseMap[filename]; ok {
		return CourseMapArray[index], nil
	}

	ef, err := xlsx.OpenFile(filename)
	if err != nil {
		fmt.Printf("err:%v\n", err)
		return nil, err
	}
	if len(ef.Sheets) == 0 {
		return nil, errors.New("excel empty sheet")
	}
	sheet := ef.Sheets[0]
	rows := sheet.Rows
	if len(rows) == 0 {
		return nil, errors.New("excel empty rows")
	}
	ret := map[string][]string{}
	for i, row := range rows {
		cells := row.Cells
		if len(cells) == 0 {
			continue
		}
		if len(cells) < 3 {
			return nil,  errors.New(fmt.Sprintf("第%d行信息不足", i+1))
		}
		questionType := cells[0].Value
		tag := getTag(questionType)
		if tag == "" {
			return nil,  errors.New(fmt.Sprintf("第%d行题目类型错误", i+1))
		}
		questionName := cells[1].Value
		questionName = TrimSpace(questionName)
		questionName = ParseHan(questionName)
		ret[tag+"-"+questionName] = []string{}
		for i := 2; i < len(cells); i++ {
			answer := cells[i].Value
			answer = TrimSpace(answer)
			if answer == "" {
				continue
			}
			ret[tag+"-"+questionName] = append(ret[questionName], answer)
		}
	}
	CourseMapArray = append(CourseMapArray, ret)
	CourseMap[filename] = len(CourseMapArray)-1
	return ret, nil
}

// 从txt 导入 execel
func LoadQuestionFile(filename string) error {


	ef, err := xlsx.OpenFile("思想道德与法治.xlsx")
	if err != nil {
		fmt.Printf("err:%v\n", err)
		return err
	}
	if len(ef.Sheets) == 0 {
		fmt.Printf("err:%v\n", "empty sheet")
		return errors.New("excel empty sheet")
	}
	fmt.Printf("start parse\n")
	sheet := ef.Sheets[0]






	m := map[string]bool{
		"一":true,
		"二":true,
		"三":true,
		"四":true,
		"五":true,
		"六":true,
		"七":true,
		"八":true,
		"九":true,
		"1":true,
		"2":true,
		"3":true,
		"4":true,
		"5":true,
		"6":true,
		"7":true,
		"8":true,
		"9":true,
		"对":true,
		"错":true,
		"A":true,
		"B":true,
		"C":true,
		"D":true,
		"E":true,
		"F":true,
		"G":true,


	}
	fi, err := os.Open(filename)
	if err != nil {
		return err
	}
	defer fi.Close()

	br := bufio.NewReader(fi)
	tag := 0
	question := ""
	questionOrigin := ""
	for {
		a, _, c := br.ReadLine()
		//fmt.Printf("xx:%s\n", a)
		if c == io.EOF {
			break
		}
		if len(a) == 0 {
			continue
		}
		line := ""
		str := string(a)
		str = strings.TrimSpace(str)
		for _, r := range str {
			if _, ok := m[string(r)]; !ok {
				break
			}
			line = str

			break
		}
		if line == "" {
			continue
		}
		//fmt.Printf("line:%s\n", line)
		if strings.Contains(line, "单选题") {
			tag = 1
			continue
		}
		if strings.Contains(line, "判断题") {
			tag = 2
			continue
		}

		if tag == 0 {
			continue
		}


		if question == "" {
			question = ParseHan(line)
			questionOrigin = string(a)
			continue
		}

		row := sheet.AddRow()
		cell := row.AddCell()
		cell.Value = "单选题"
		if tag == 2 {
			cell.Value = "判断题"
		}
		cell = row.AddCell()
		cell.Value = questionOrigin
		cell = row.AddCell()
		cell.Value = line
		fmt.Printf("题目：%v\n", question)
		fmt.Printf("答案：%v\n", line)
		question = ""
		//break
	}
	ef.Save("思想道德与法治.xlsx")

	return nil
}
